#!/usr/bin/env python
# encoding: utf-8
"""
$Id$

Purpose: Run the ATMS SDR using Raytheon ADL 3.

Input:
    One or more HDF5 ATMS RDR input files, aggregated or single-granule.
    You need one granule before and one granule after a given granule to process.
    A work directory, typically, empty, in which to unpack the granules and generate the output.
    If the work directory specified does not exist, it will be created.

Output:
    ADL ATMS SDR blob files, .lutasc metadata files, and HDF5 output granules will be created.

Details:
    If you input a series of granules, the software will scan the work directory.
    Thus, for N input RDR granules, you may get up to N-2 output SDR granules if they are all contiguous.
    It is ambiguous to provide several copies of the same granule in the work directory;
    this will result in an error abort.
    The unpacker gives each unpacking of a given granule its own

Preconditions:
    Requires ADL_HOME, CSPP_RT_ANC_PATH, CSPP_RT_ANC_CACHE_DIR environment variables are set.
    Requires that any needed LD_LIBRARY_PATH is set.
    Requires that DSTATICDATA is set.

Optional:
    Environment variables CSPP_RT_ANC_PATH, CSPP_RT_ANC_CACHE_DIR, if static ancillary
     data is not placed within ADL_HOME.

Created Oct 2011 by R.K.Garcia <rayg@ssec.wisc.edu>
Copyright (c) 2011-2012 University of Wisconsin Regents.
Licensed under GNU GPLv3.
"""

import os
import sys
import logging
import glob
import traceback
import time

import datetime as dt
from subprocess import CalledProcessError
from shutil import rmtree
from pprint import pprint
from multiprocessing import Pool
import shutil
# skim and convert routines for reading .lutasc metadata fields of interest
from adl_asc import contiguous_granule_groups, granule_groups_contain, effective_anc_contains, \
    RDR_REQUIRED_KEYS, POLARWANDER_REQUIRED_KEYS
from adl_query_dms import skim_dir, setup_dms_in_work_directory,check_spacecraft_diaries
# post-processing on blob+lutasc products
from adl_post_process import repack_products, aggregate_products_par, add_geo_attribute_to_aggregates
from adl_post_process import SHORTNAME_2_PRODUCTID
import rdr_unpack

# ancillary search and unpacker common routines
from adl_common import sh, anc_files_needed, link_ancillary_to_work_dir, configure_logging, \
    env, h5_xdr_inventory, get_return_code, check_env ,CSPP_RT_HOME, check_static
from adl_common import check_anc_cache, ADL_HOME,  CSPP_RT_ANC_CACHE_DIR, COMMON_LOG_CHECK_TABLE, \
    check_and_convert_path, status_line, check_existing_env_var,initialize_work_ADL_variables,ADL_VARS
from sdr_common import abs_filenames
from sdr_common import cleanup_work_dir
from sdr_common import add_geo_attribute_to_h5,rename_products
# log file scanning
import adl_log
from adl_atms_data import ADL_ATMS_ANC_GLOBS, GIG_MEMORY_PER_PROC, ADL_ATMS_PRODUCT_IDS, ATMS_GRAN_ID_DELTA

# cache loading and searching
# N_GEO_Ref fix for ADL
from adl_geo_ref import write_geo_ref

from sdr_common import check_products,  build_workspace, setup_multi_process, trim_the_granules, \
    start_out_good, report_gran_completion, number_good_granules, check_memory, check_number_processors, \
    move_products_to_work_directory

import adl_anc_retrieval




# every module should have a LOG object
LOG = logging.getLogger('adl_atms_sdr')

ADL_ATMS_SDR = os.path.join(ADL_HOME, 'bin', 'ProSdrAtmsController.exe')

# keys used in metadata dictionaries
K_FILENAME = '_asc_filename'

# directories in which we find the ancillary files, including CC-Int file for ATMS SDR
#ADL_ANC_DIRS = CSPP_RT_ANC_PATH

# filenames which we'll need to set N_GEO_Ref on
# FIXME: this should be derived from product lists and not from filesystem search?
ADL_ATMS_SDR_GLOBS = ('SATMS*.h5', 'TATMS*.h5')

# we're not likely to succeed in processing using granules smaller than this many bytes
MINIMUM_RDR_BLOB_SIZE = 100000

MAX_CONTIGUOUS_DELTA_CRIS_RTSTPS_5_3 = 30  # in seconds; this accounts for defects in RT-STPS 5.3

# attribute paths for SDRs and TDRs
# FIXME: review this
SDR_GRANULE_ID_ATTR_PATH = 'Data_Products/ATMS-SDR/ATMS-SDR_Gran_0/N_Granule_ID'
TDR_GRANULE_ID_ATTR_PATH = 'Data_Products/ATMS-TDR/ATMS-TDR_Gran_0/N_Granule_ID'

# products which we aggregate
ATMS_SDR_PRODUCTS_AGG = [
    'ATMS-SDR',
    # 'ATMS-FSDR',
    'ATMS-TDR',
    'ATMS-SDR-GEO'
]

# products which we aggregate and assign N_GEO_Ref to
ATMS_SDR_PRODUCTS_AGG_GEO = [
    'ATMS-SDR',
    'ATMS-TDR'
]

# intermediate products we're not really doing anything with
ATMS_SDR_PRODUCTS_NOAGG = [
    'ATMS-SDR-IP'
]

XML_TMPL_ATMS_SDR = """
<?xml version="1.0" encoding="UTF-8"?>
<!--
  Config file to run a WFM-managed process in lightweight (no WFM) mode.

  Run a WFM-managed process with no args for help on the format of this file.
-->
<InfWfm_TkLwConfig xmlns="http://jpss.lwtkcfg.ray.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://jpss.lwtkcfg.ray.com ExecutionConfig.xsd">
  <processName>ProSdrAtmsController.exe</processName>
  <perfEnable>FALSE</perfEnable>
  <perfPath>${GRANULE_OUTPUT_DIR}</perfPath>
  <debugDest>D_FILE</debugDest>
  <debugLevel>DBG_HIGH</debugLevel>
  <debugPath>${GRANULE_OUTPUT_DIR}</debugPath>
  <rootDir>${GRANULE_OUTPUT_DIR}</rootDir>
  <startMode>INF_STARTMODE_COLD</startMode>
  <healthInterval>30</healthInterval>
  <configGuideName>ProSdrAtmsController_GuideList.cfg</configGuideName>
"""


XML_TMPL_ATMS_SDR_full = """
<?xml version="1.0" encoding="UTF-8"?>
<!--
  Config file to run a WFM-managed process in lightweight (no WFM) mode.

  Run a WFM-managed process with no args for help on the format of this file.
-->
<InfWfm_TkLwConfig xmlns="http://jpss.lwtkcfg.ray.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://jpss.lwtkcfg.ray.com ExecutionConfig.xsd">
  <processName>ProSdrAtmsController.exe</processName>
  <perfEnable>FALSE</perfEnable>
  <perfPath>${GRANULE_OUTPUT_DIR}</perfPath>
  <debugDest>D_FILE</debugDest>
  <debugLevel>DBG_HIGH</debugLevel>
  <debugPath>${GRANULE_OUTPUT_DIR}</debugPath>
  <rootDir>${GRANULE_OUTPUT_DIR}</rootDir>
  <startMode>INF_STARTMODE_COLD</startMode>
  <healthInterval>30</healthInterval>
  <configGuideName>ProSdrAtmsController_GuideList.cfg</configGuideName>

  <task>
    <taskType>GRANULE</taskType>
    <granuleId>%(N_Granule_ID)s</granuleId>
    <granuleVersion>A1</granuleVersion>
    <spacecraft>${SPCRFT}</spacecraft>
    <sensor>ATMS</sensor>
  </task>
 
  <task>
    <taskType>SHUTDOWN</taskType>
  </task>
</InfWfm_TkLwConfig>

"""

def _skim_atms_science_rdr(work_dir):
    "skim for ATMS-SCIENCE-RDR data meeting minimum requirements"
    for info in skim_dir(required_keys=RDR_REQUIRED_KEYS, n_collection_short_name="ATMS-SCIENCE-RDR"):
        path = info['BlobPath']
        if not os.path.isfile(path) or os.stat(path).st_size < MINIMUM_RDR_BLOB_SIZE:
            LOG.info('Ignoring %s, invalid blob. For direct broadcast this can happen at start or end of pass.' % path)
        else:
            yield info


def sift_metadata_for_atms_sdr(spcrft, work_dir='.'):
    """
    search through the ASC metadata in a directory,
    grouping in StartTime order
    look for back-to-back granules and break into contiguous sequences
    check that S/C diary RDRs are available for granules of interest
    yield sequence of granules we can process
    """

    Science_RDRs = list(skim_dir(n_collection_short_name="ATMS-SCIENCE-RDR", spcrft=spcrft))

    status_line("Total Science RDRs: " + str(len(Science_RDRs)))
    start_time_key = lambda x: (x['StartTime'], x.get('N_Granule_Version', None))

    for group in sorted(Science_RDRs, key=start_time_key):
        yield group

    return


"""
    LOG.debug('collecting information for S/C diary RDRs')
    # FIXME: a bug in RT-STPS incremental RDR granule production makes a
    # short S/C Diary RDR at the start of each natural boundary, which we want to ignore for now
    # eventually the larger_granules_preferred option should go away
    diaries = list(contiguous_granule_groups(skim_dir(work_dir,
                                                      required_keys=RDR_REQUIRED_KEYS,
                                                      n_collection_short_name='SPACECRAFT-DIARY-RDR'),
                                             larger_granules_preferred=True))
    if len(diaries) == 0:
        LOG.error('No spacecraft diary files were found!')




    LOG.debug('sifting science RDR data for processing opportunities')
    for group in contiguous_granule_groups(_skim_atms_science_rdr(work_dir),
                                           tolerance=dt.timedelta(seconds=MAX_CONTIGUOUS_DELTA_CRIS_RTSTPS_5_3)):
        # for ATMS, we can process everything but the first and last granule
        # for CrIS, use [4:-4]
        LOG.debug('contiguous granule group: %r' % (group,))
        for gran in group[1:-1]:
            if not granule_groups_contain(diaries, gran):
                LOG.info("Insufficient S/C Diary RDR coverage to process %s @ %s (%s) - skipping" % (
                    gran['N_Granule_ID'], gran['StartTime'], gran['URID']))
                continue
            LOG.debug('Processing opportunity: %r at %s with uuid %s' % (
                gran['N_Granule_ID'], gran['StartTime'], gran['URID']))
            yield gran
"""


# Look through new log files for completed messages
# based on CrIS SDR original
def check_log_files(work_dir, pid, xml):
    """
    Find the log file
    Look for success
    Return True if found
    Display log message and hint if problem occurred
    """
    # retrieve exe name and log path from lw file
    logpat = os.path.join(work_dir, "log", "*%d*.log" % pid)

    zult = True
    for logFile in glob.glob(logpat):
        LOG.debug("Checking Log file " + logFile + " for errors.")
        n_err = adl_log.scan_log_file(COMMON_LOG_CHECK_TABLE, logFile)

        if n_err == 0:
            LOG.debug(" Log: " + logFile + " " + xml + " Completed successfully")
        else:
            LOG.error(" Log: " + logFile + " " + xml + " Completed unsuccessfully")
            zult = False

    return zult


def products_in_directory(work_dir, product_ids):
    """ Checks that proper products were produced.
    If product h5 file was produced blob and lutasc files are deleted
    If product was not produced files are left alone
    """

    for product_id in product_ids:
        files = glob.glob(os.path.join(work_dir, '%s*.h5' % product_id))
        for afile in files:
            yield afile
    return


def run_and_check_single_granule(additional_env):
    "run a VIIRS SDR XML input in sequence"
    good = True

    work_dir = additional_env['WORK_DIR']
    xml = additional_env['XML_FILE']
    granule_id = additional_env['N_Granule_ID']
    granule_output_dir = additional_env['ADL_HDF_OUTPUT_PATH']
    chunk_size = int(additional_env['CHUNK_SIZE'])
    chunks = 0
    satpat = os.path.join(granule_output_dir, 'SATMS*.h5')
    tatpat = os.path.join(granule_output_dir, 'TATMS*.h5')

    _, tatms = h5_xdr_inventory(tatpat, TDR_GRANULE_ID_ATTR_PATH)
    prior_granules, satms = h5_xdr_inventory(satpat, SDR_GRANULE_ID_ATTR_PATH)

    cmd = [ADL_ATMS_SDR, xml]

    LOG.debug('executing "%s"' % ' '.join(cmd))
    LOG.debug('additional environment variables: %s' % repr(additional_env))

    start_time = time.time()


    try:

        z = ADL_VARS.copy()
        z.update(additional_env)  # which returns None since it mutates z
        pid = sh(cmd, env=env(**z), cwd=work_dir)
        LOG.debug("%r ran as pid %d" % (cmd, pid))
    # if not check_log_files(work_dir, pid, xml):
    #            bad_log_runs.add(granule_id)

    except CalledProcessError as oops:
        pid = getattr(oops, 'pid', None)
        #        if not check_log_files(work_dir, pid, xml):
        #            bad_log_runs.add(granule_id)

        LOG.debug(traceback.format_exc())
        LOG.error('%s failed on %r: %r. Continuing...' % (cmd[0], xml, oops))
        #        crashed_runs.add(granule_id)
        good = False

    product_shortnames = ATMS_SDR_PRODUCTS_AGG + ATMS_SDR_PRODUCTS_AGG_GEO

    problem = check_products(granule_output_dir, granule_id, product_shortnames,chunk_size=chunk_size)

    if problem is True:
        good = False
      
      
    rename_products(granule_output_dir, ADL_ATMS_PRODUCT_IDS)
  
    gran_id = additional_env['N_Granule_ID']

    add_geo_attribute_to_h5(
        granule_output_dir, gran_id, ATMS_SDR_PRODUCTS_AGG)

    chunks , option_chunks = move_products_to_work_directory(granule_output_dir, ADL_ATMS_PRODUCT_IDS)

    end_time = time.time()
    elapsed_time = end_time - start_time
    status_line('Tot Execution Time:  "%f" Gran "%s"' %
                (elapsed_time, granule_id))

    stat = ""
    if good is True:
        stat = 'Granule %s Completed without problems. ' % granule_id

    if good is False:
        stat = 'Granule %s Completed with problems. ' % granule_id

    report_gran_completion(stat,chunks,int(additional_env["ACTUAL"]))

    result = {"GOOD": str(good)}

    return result


def run_all_granules(work_dir, granules_to_process, trim=None, setup_only=False, CLEANUP="True", nprocs="1",
                     spcrft='NPP',accuracyMode='reduced',
                     **additional_env):
    """run each ATMS SDR XML input in sequence
    return the list of granule IDs which crashed, and list of granule ids which did not create output
    """
    if trim is None:
        trim = [0, 0]

    global total_granules

    noncritical_problem = 0
    # if tatms or satms:
    #     LOG.warning("This working directory already
    # has products in it! Starting with a clean working directory is strongly recommended.")

    total_granules = len(granules_to_process)
    #LOG.info('%d granules to process' % (len(granules_to_process)))

    granules_to_process = trim_the_granules(granules_to_process, ATMS_GRAN_ID_DELTA, trim=trim)

    fnxml = 'sdr_atms_%s.xml'
    chunk_size= int( len(granules_to_process) / int (nprocs) )
    if chunk_size <= 0:
        chunk_size=1
    chunk_size=1
    additional_keys = dict(SPCRFT=spcrft,INSTRUMENT='ATMS',CHUNK_SIZE=str(chunk_size))

    argument_dictionaries, granules_to_process = setup_multi_process(work_dir,
                                                                     granules_to_process,
                                                                     [fnxml],
                                                                     [XML_TMPL_ATMS_SDR],
                                                                     
                                                                     additional_keys,accuracyMode=accuracyMode)

    total_granules = len(granules_to_process)
    start_out_good(total_granules)

    keep_all_output = False
    results = []
    # results = map (run_and_check_single_viirs_sdr, argument_dictionaries )
    if len(argument_dictionaries) > 0:

        nprocs = check_number_processors(int(nprocs))
        gig_recomended = GIG_MEMORY_PER_PROC
        check_memory(nprocs, gig_recomended)

        parallel = Pool(int(nprocs))

        try:
            results = parallel.map_async(run_and_check_single_granule, argument_dictionaries).get(9999999)
        except KeyboardInterrupt:
            LOG.error("Got exception, stopping workers and exiting.")
            parallel.terminate()
            parallel.join()
            sys.exit(1)

    for result in results[1:-1]:
        # do not fail on first or last granule - these can have errors
        if result["GOOD"] != "True":
            LOG.debug("result check %s" % result["GOOD"])
            keep_all_output = True
            noncritical_problem += 1

    if CLEANUP == str(True) and keep_all_output is False:
        for args in argument_dictionaries:
            granule_dir = args['GRANULE_OUTPUT_DIR']
            LOG.debug("Remove: " + granule_dir)
            if os.path.exists(granule_dir):
                shutil.rmtree(granule_dir)

                # clean up the science data only if you there are no other granule directories left

        remaining_files_and_directories = glob.glob(os.path.join(work_dir, "NPP*"))

        if len(remaining_files_and_directories) == 0:
            log_dir = os.path.join(work_dir, "log")
            db = os.path.join(work_dir, "db")
            cleanup_work_dir(work_dir)
    return noncritical_problem



def check_ancillary_effectivity(granules_to_process, anc_dir):
    "verify that we have enough polarwander coverage on all granules, returning filtered list of granules to process"
    return granules_to_process


def atms_sdr(work_dir, h5_names, trim=None, setup_only=False, allow_cache_update=True,
             debug=False, compress=True, aggregate=True, 
             nprocs="1", spcrft='NPP',accuracyMode='reduced',center=False):
    "process ATMS RDR data in HDF5 format (any aggregation) to SDRs in an arbitrary work directory"

    if not trim:
        trim = [0, 0]

    build_workspace(work_dir)

    unpacking_problems = rdr_unpack.unpack_rdr(work_dir, h5_names, allow_cache_update=allow_cache_update)
    check_spacecraft_diaries(spcrft=spcrft)
    # read through ascii metadata and build up information table
    #LOG.info('sifting through metadata to find ATMS SDR processing candidates')

    granules_to_process = list(sift_metadata_for_atms_sdr(spcrft, work_dir))
    
    LOG.debug(', '.join(x['N_Granule_ID'] for x in granules_to_process))
    if not granules_to_process:
        LOG.warning("Found no granules to process!")
        return get_return_code(unpacking_problems, num_xml_files_to_process=0, num_no_output_runs=0,
                               noncritical_problem=False, environment_error=0)

    # test mode to pick center granules for shortened test cycle will result in one  granule processed
    if center is True: 
        c = int(len(granules_to_process)/2)
        r = int(len(granules_to_process) % 2)
        trim[0] = c
        trim[1] =   ( c - 1  + r )
        #LOG.info(' r %d c %d o %d '%(r,c,(len(granules_to_process) - ( c - r ))))
        LOG.debug(trim)


    xml_files_to_process = []
    try:
        
        adl_anc_retrieval.service_remote_ancillary(
                                    work_dir, granules_to_process,
                                    adl_anc_retrieval.kPOLAR,spcrft,
                                    allow_cache_update=allow_cache_update)
        
        adl_anc_retrieval.service_remote_ancillary(
                                    work_dir, granules_to_process,
                                    adl_anc_retrieval.kTLE,spcrft,
                                    allow_cache_update=allow_cache_update)
                                    
        adl_anc_retrieval.update_orbit_in_db(work_dir,granules_to_process,spcrft)

        # granules_to_process = check_ancillary_effectivity(granules_to_process, anc_dir)

        # build XML configuration files for jobs that can be run

        LOG.info("running SDR on XML files")
        cleanup = str(not debug)
        noncritical_problem = run_all_granules(work_dir, granules_to_process, trim=trim, setup_only=setup_only,
                                               CLEANUP=cleanup, WORK_DIR=work_dir,
                                               ADL_HOME=ADL_HOME,
                                               nprocs=nprocs, spcrft=spcrft,accuracyMode=accuracyMode)
        
        #rename_products(work_dir, ADL_ATMS_PRODUCT_IDS)

        # considered a noncritical problem if there were any crashed runs, runs that produced no output,
        # runs where Geo failed, or runs where ADL logs indicated a problem
        #        noncritical_problem = crashed_runs or no_output_runs or geo_problem_runs or bad_log_runs

        product_shortnames = ATMS_SDR_PRODUCTS_AGG + ATMS_SDR_PRODUCTS_AGG_GEO
        # FIXME: move this to cris_sdr and should add return codes to aggregate_products() and repack_products()
        if aggregate is True:
            aggregate_products_par(work_dir, ATMS_SDR_PRODUCTS_AGG, nprocs=str(nprocs),spcrft=spcrft)
            add_geo_attribute_to_h5(work_dir, "aggregates", ATMS_SDR_PRODUCTS_AGG_GEO)
        if compress is True:
            repack_products(work_dir, product_shortnames)

        #

        # print final disposition message and get return code
        total_granules = len(xml_files_to_process)
        gran_failure = total_granules - number_good_granules()
        rc = get_return_code(unpacking_problems, total_granules, gran_failure, noncritical_problem,
                             environment_error=False)

    except EnvironmentError:
        LOG.error(traceback.format_exc())

        # print final disposition message and get return code
        rc = get_return_code(unpacking_problems, len(xml_files_to_process), num_no_output_runs=0,
                             noncritical_problem=False, environment_error=True)


    return rc


def main():
    import argparse

    desc = """Build ATMS SDR ProSdrAtmsController work directory and run ATMS SDR."""
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument('-T', '--test',
                        action="store_true", default=False, help="run self-tests")
    parser.add_argument('-l', '--local',
                        action="store_true", default=False, help="disable download of remote ancillary data to cache")
    parser.add_argument('-W', '--work-dir', metavar='work_dir', default='.',
                        help='directory in which all activity occurs, defaults to current dir')
    parser.add_argument('-t', '--trim', nargs=2, type=int, metavar='N',
                        default=None,
                        help='Trim granules. -t \"Trim Begin\" \"Trim End\" -t 1 1 Trims first and last granule.')

    parser.add_argument('-m', '--accuracymode',
                        action="store_true", default=False, help="increase geo accuracy")

    parser.add_argument('--center', dest='center', action='store_true',
                        help='Trim to center granule')
    parser.add_argument('--no-center', dest='center', action='store_false',
                        help='Normal output')
    parser.set_defaults(center=False)
    
    parser.add_argument('-z', '--zip',
                        action="store_true", default=False, help="compress products with h5repack zip compression")
    parser.add_argument('-a', '--aggregate',
                        action="store_true", default=False, help="aggregate products with nagg")
    parser.add_argument('-d', '--debug', action="store_true", default=False,
                        help='enable debug mode on ADL and avoid cleaning workspace')
    parser.add_argument('-v', '--verbosity', action="count", default=0,
                        help='each occurrence increases verbosity 1 level through ERROR-WARNING-INFO-DEBUG')
    parser.add_argument('filenames', metavar='filename', type=str, nargs='+',
                        help='HDF5 ATMS RDR file/s to process')

    parser.add_argument('-p', '--processor',
                        type=int, default=1, help="Number of processors to use for granule processing")

    args = parser.parse_args()

    levels = [logging.ERROR, logging.WARN, logging.INFO, logging.DEBUG]
    level = levels[args.verbosity if args.verbosity < 4 else 3]
    work_dir = check_and_convert_path("WORK_DIR", args.work_dir)
    d = dt.datetime.now()
    timestamp = d.isoformat()
    logname = "atms_sdr." + timestamp + ".log"
    logfile = os.path.join(work_dir, logname)
    configure_logging(level, FILE=logfile)

#     if args.aggregate is True:
#         if len(args.filenames) > 1:
#             LOG.error("Only one input allowed with aggregate option")
#             sys.exit(2)

    dpe_version = check_existing_env_var('DPE_VER', default_value=None)
    LOG.info("product version: DPE_VER for ATMS SDR  %s" % dpe_version)

    work_dir = os.path.abspath(args.work_dir)

    LOG.info('CSPP execution work directory is %r' % work_dir)

    if not args.trim:
        args.trim = [0, 0]

    if not args:
        parser.print_help()
        return 9
    
    accuracyMode='reduced'
    if args.accuracymode is True:
        accuracyMode = 'full'


    
    check_anc_cache(check_write=not args.local)
    check_static()
    full_db_name = setup_dms_in_work_directory(work_dir, CSPP_RT_HOME)
    initialize_work_ADL_variables(work_dir,full_db_name)

    check_env()

    rc = 1
    try:
        if 'npp' in os.path.basename( args.filenames[0]):
            spcrft = 'NPP'
        else:
            spcrft = 'J01'

        filenames = abs_filenames(args.filenames)

        rc = atms_sdr(work_dir, filenames, allow_cache_update=not args.local,
                      debug=args.debug, trim=args.trim, aggregate=args.aggregate, compress=args.zip,
                      nprocs=str(args.processor), spcrft=spcrft,accuracyMode=accuracyMode,center=args.center)

        if not args.debug:
            os.remove(logfile)
            cleanup_work_dir(work_dir,force_all=True,spcrft=spcrft)
            

    except Exception:
        LOG.error(traceback.format_exc())

    return rc


if __name__ == '__main__':
    sys.exit(main())
